#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <usbdrv.h>
#define USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH 63

/* ------------------------------------------------------------------------- */
typedef struct 
{
	uint8_t modifier;  // แป้น Shift, Ctrl, Alt
	uint8_t keycode;   // แป้นทั่วไป
} keyboard_report_t;

keyboard_report_t reportBuffer; /* buffer for HID reports */
uchar idleRate = 10;            /* in 4 ms units */

/* ------------------------------------------------------------------------- */

PROGMEM const char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = 
{ /* USB report descriptor */
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};
/* We use a simplifed keyboard report descriptor which does not support the
 * boot protocol. We don't allow setting status LEDs and we only allow one
 * simultaneous key press (except modifiers). We can therefore use short
 * 2 byte input reports.
 * The report descriptor has been created with usb.org's "HID Descriptor Tool"
 * which can be downloaded from http://www.usb.org/developers/hidpage/.
 * Redundant entries (such as LOGICAL_MINIMUM and USAGE_PAGE) have been omitted
 * for the second INPUT item.
 */

/* Keyboard usage values, see usb.org's HID-usage-tables document, chapter
 * 10 Keyboard/Keypad Page for more codes.
 */
#define MOD_CONTROL_LEFT    (1<<0)
#define MOD_SHIFT_LEFT      (1<<1)
#define MOD_ALT_LEFT        (1<<2)
#define MOD_GUI_LEFT        (1<<3)
#define MOD_CONTROL_RIGHT   (1<<4)
#define MOD_SHIFT_RIGHT     (1<<5)
#define MOD_ALT_RIGHT       (1<<6)
#define MOD_GUI_RIGHT       (1<<7)

/*
 * เตรียมข้อความที่จะส่งไปยังโฮสท์
 * keycode ดึงมาจากเอกสาร Universial Serial Bus (USB) HID Usage Tables
 */
uint8_t output[] = {
	
    0x1E,  // keycode 't'
    0x1F,  // keycode 'i'
    0x20,  // keycode 'down'
    0x21,  // keycode 'up'
    0x22,  // keycode 'left'
    0x23,  // keycode 'right' 
    0x24,  // keycode 'u'
    0x25,  // keycode 'm'
   
};

/* ------------------------------------------------------------------------- */
/* ------------------------ interface to USB driver ------------------------ */
/* ------------------------------------------------------------------------- */

uchar usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;

    if ((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS)
    {
        /* class request type */
        if (rq->bRequest == USBRQ_HID_GET_REPORT)
        {
            /* we only have one report type, so don't look at wValue */
            reportBuffer.modifier = 0;
            reportBuffer.keycode  = 0;
            usbMsgPtr = (void*)&reportBuffer;
            return sizeof(reportBuffer);
        }
        else if (rq->bRequest == USBRQ_HID_GET_IDLE)
        {
            usbMsgPtr = &idleRate;
            return 1;
        }
        else if (rq->bRequest == USBRQ_HID_SET_IDLE)
        {
            idleRate = rq->wValue.bytes[1];
        }
    }
	return 0;
}

void sendCode(uint8_t modifier, uint8_t keycode)
{
    reportBuffer.modifier = modifier;
    reportBuffer.keycode = keycode;
    usbSetInterrupt((uchar*)&reportBuffer, sizeof(reportBuffer));
}

/* ------------------------------------------------------------------------- */
/* --------------------------------- main ---------------------------------- */
/* ------------------------------------------------------------------------- */

int main(void)
{
    uint8_t index = 0;
	
	
	DDRB = 0b00000000;   //brake(0)-accelerate(4)
    DDRC  = 0b00000000;  //turnleft(0)right(5)
    PORTC = (1 << PC0)|(1<<PC5);  // Enable pull-up at PC0 5*/
	DDRD &= 0b10011100;  //extra button(05   61)
	PORTD = (1 << PD0)|(1 << PD1)|(1 << PD5)|(1 << PD6);  //enable pull-up at PD3
	
	
	

	
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    usbInit();

    sei();


    for(;;)
    {
        do { usbPoll(); } while (!usbInterruptIsReady());
        sendCode(0,0); //for releasing buttons
        
        
        if(( (PINB>>PB0) & 0b00000001) == 1) //If press -> value=1
        {
        	do 
        	{ 
        		usbPoll();
        	} while (!usbInterruptIsReady());
        	sendCode(0,output[0]);
        }
        if(( (PINB>>PB4) & 0b00000001) == 1)  
        {
        	do 
        	{ 
        		usbPoll();
        	} while (!usbInterruptIsReady());
        	sendCode(0,output[1]);
        }
        if(( (PINC>>PC0) & 0b00000001) == 0) 
        {
        	do 
        	{ 
        		usbPoll();
        	} while (!usbInterruptIsReady());
        	sendCode(0,output[2]);
        }
        if(( (PINC>>PC5) & 0b00000001) == 0) 
        {
        	do 
        	{ 
        		usbPoll();
        	} while (!usbInterruptIsReady());
        	sendCode(0,output[3]);
        }
        if(( (PIND>>PD0) & 0b00000001) == 0) //If press -> value =0(as pull up was set)
        {
        	do 
        	{
        		usbPoll();	
        	} while (!usbInterruptIsReady());
        	sendCode(0,output[4]);
        }
        if(( (PIND>>PD1) & 0b00000001) == 0) 
        {
        	do 
        	{ 
        		usbPoll();
        	} while (!usbInterruptIsReady());
        	sendCode(0,output[6]);
        }
        if(( (PIND>>PD5) & 0b00000001) == 0) 
        {
        	do 
        	{
        		usbPoll();
        	} while (!usbInterruptIsReady());
        	sendCode(0,output[5]);
        }
        if(( (PIND>>PD6) & 0b00000001) == 0) 
        {
        	do 
        	{
        		usbPoll();
        	} while (!usbInterruptIsReady());
        	sendCode(0,output[7]);
        }   
        
       
    }

    return 0;
}
